<?php

/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2022 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @author     Malin Lorch <mlorch@liquidweb.com>
 * @date       May 25, 2022
 */


/**
 * An application is a backup plan specific to the resource.
 *
 * It is a copy of a policy. Only one application should exist per policy
 * for a specific resource of a specific type.
 */
class Model_Api_Public_Policies extends Model_Api
{
    const API_ENDPOINT_POLICIES = '/api/policy_management/v4/policies';
    const API_ENDPOINT_POLICY_STATUSES = '/api/policy_management/v4/policy_statuses';

    const TYPE_POLICY_BACKUP_MACHINE = 'policy.backup.machine';
    const TYPE_POLICY_PROTECTION_TOTAL = 'policy.protection.total';
    const TYPE_POLICY_BACKUP_INVALID = 'policy.backup.invalid';

    const TYPE = 'type';
    const POLICY = 'policy';

    const APPLICABLE_CONTEXT = 'applicable_to_context_id';

    private $_backup_plan_id;
    private $_total_plan_id;

    /**
     * @param string $policy_id
     */
    public function getTotalPolicy($policy_id): Model_Backups_Plan_Policy_Total
    {
        $query = http_build_query(array(
            'include_applied_context' => true, // include resource ids for resources using this plan
            'full_composite' => true, // include child policies (e.g. policy.backup.machine)
            'include_settings' => true, // include settings, important for backup machine policy, for pre/post script and schedule
        ));

        $apiEndpoint = self::API_ENDPOINT_POLICIES . "/$policy_id?$query";
        $res = $this->_getOAuthClient()->sendGetRequest($apiEndpoint);
        IW::getLog()->debug('Acronis: attempted to fetch total protection plan policy', $res);

        $policies = $res['policy'];

        if (empty($policies)) {
            throw new Exception_Acronis_BackupPlan_Missing("Plan with id $policy_id does not exist!");
        }

        $context = $res['context'];
        return new Model_Backups_Plan_Policy_Total($policies, $context);
    }

    /**
     * @param string $policy_id
     */
    public function getBackupMachinePolicy($policy_id): Model_Backups_Plan_Policy_BackupMachine
    {
        // putting id as query instead of param ensures that parent is retrieved as well

        $query = http_build_query(array(
            'include_applied_context' => true, // include resource ids for resources using this plan
            'policy_id' => $policy_id, // include child policies (e.g. policy.backup.machine)
            'include_settings' => true, // include settings, important for backup machine policy, for pre/post script and schedule
        ));

        $apiEndpoint = self::API_ENDPOINT_POLICIES . "?$query";
        $res = $this->_getOAuthClient()->sendGetRequest($apiEndpoint);

        IW::getLog()->debug('Acronis: attempted to fetch backup machine policy', $res);

        // should only get one item - because of query
        $res = $res['items'][0];
        $policies = $res['policy'];

        if (empty($policies)) {
            throw new Exception_Acronis_BackupPlan_Missing("Plan with id $policy_id does not exist!");
        }

        $context = $res['context'];
        return new Model_Backups_Plan_Policy_BackupMachine($policies, $context);
    }

    /**
     * Get applicable plans/ existing plans not assigned to the resource.
     *
     * Also called Policies.
     *
     * @return Model_Backups_Plan_Policy_Total[]
     */
    public function getApplicableRootPolicies()
    {
        $query = http_build_query(array(
            'applicable_to_context_id' => $this->_getInstanceId(),
            'include_settings' => true, // gets pre/post script settings
        ));

        $endpoint = self::API_ENDPOINT_POLICIES . "?$query";
        $api_res = $this->_getOAuthClient()->sendGetRequest($endpoint);
        $items = $api_res['items'];

        if ($items === null) {
            return array();
        }

        return $this->_mapPolicies($items);
    }

    /**
     * @param array $api_res
     * @return Model_Backups_Plan_Policy_Total[]
     */
    private function _mapPolicies(array $api_res): array
    {
        return array_map(function ($plan) {
            $policies = $plan[self::POLICY];
            $context = $plan['context'];

            return  new Model_Backups_Plan_Policy_Total($policies, $context);
        }, $api_res);
    }

    /**
     * Get all root policies applied to the resource.
     *
     * @return Model_Backups_Plan_Policy_Total[]
     */
    public function getAppliedRootPolicies(): array
    {
        $items = $this->_getAppliedPolicies();

        if (is_null($items)) {
            return array();
        }

        return $this->_filterAppliedPolicies($items);
    }

    /**
     * @param array $api_res
     */
    private function _filterAppliedPolicies(array $api_res)
    {
        $applied = array();

        foreach ($api_res as $plan) {
            $policies = $plan['policy'];
            $context = $plan['context'];

            $Policy = new Model_Backups_Plan_Policy_Total($policies, $context);

            if ($Policy->isProtectingResource($this->_getInstanceId())) {
                $applied[] = $Policy;
            }
        }

        return $applied;
    }

    /**
     * Get applied policies
     *
     * @return array
     */
    private function _getAppliedPolicies(): array
    {
        $query = http_build_query(array(
            'include_applied_context' => true,
            'include_settings' => true, // gets pre/post script settings
        ));

        $endpoint = self::API_ENDPOINT_POLICIES . "?$query";
        $policies = $this->_getOAuthClient()->sendGetRequest($endpoint)['items'];

        if (is_null($policies)) {
            return array();
        }

        return $policies;
    }

    /**
     * Create default plan.
     */
    public function createDefaultPolicy()
    {
        IW::getLog()->debug("Acronis: Creating default plan");
        $data = $this->_getCreatePolicyData();
        $this->_getOAuthClient()->sendPostRequest(self::API_ENDPOINT_POLICIES, $data);

        return $this->_total_plan_id;
    }

    /**
     * @return array
     */
    private function _getCreatePolicyData(): array
    {
        $create_policy_request_data = file_get_contents("/home/interworx/plugins/acronis/data_templates/default_plan.json");
        $data_json = $this->_insertIdsPoliciesRequestTemplate($create_policy_request_data);
        $data = json_decode($data_json, true);

        return $data;
    }

    /**
     * Replace template ids with real values.
     *
     * @param string $template
     * @return string
     */
    private function _insertIdsPoliciesRequestTemplate($template)
    {
        $this->_backup_plan_id = IWorxSecUtil::getUuid();
        $this->_total_plan_id = IWorxSecUtil::getUuid();

        // format {{my-variable}} correlates to postman variables
        // so that we can copy-paste template content and reuse in postman
        $template = str_replace('{{context_id}}', $this->_getInstanceId(), $template);
        $template = str_replace('{{backup_plan_id}}', $this->_backup_plan_id, $template);
        $template = str_replace('{{total_protection_plan_id}}', $this->_total_plan_id, $template);

        return $template;
    }
}
