<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2021 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @author     Malin Lorch <mlorch@liquidweb.com>
 * @date       Aug 30, 2021
 */
require_once(__DIR__ . '/../../Nodeworx.php');

/**
 * @package InterWorx
 * @subpackage
 */
class NwActivatePlanTest extends Acronis_NodeworxTest
{
    const PLAN_NO_PRESCRIPT = 'NOT DEFAULT'; // MAKE SURE THIS EXISTS IN CLOUD

    /**
     * test revokePlanCommitAction
     */
    public function testRevokePlan()
    {
        if ($this->_isActivated()) {
            echo __FUNCTION__;
            $Reply = $this->_sendCtrlRequest('revokePlan', array());
            $this->assertReplySuccessful($Reply);
            $this->_awaitRevoked();
        }
    }

    /**
     * Await revoked.
     */
    private function _awaitRevoked()
    {
        $attempts = 0;
        $maxAttempts = 10;

        while ($this->_isActivated() && $attempts < $maxAttempts) {
            sleep(10);
            $attempts++;
        }

        if ($this->_isActivated()) {
            $this->fail('AWAIT REVOKE - PLAN NOT REVOKED!');
        }
    }

    /**
     * Test listCompatiblePlansCommitAction
     */
    public function testGetBackupPlanDetailsShouldSayDisabled()
    {
        echo __FUNCTION__;

        $Reply = $this->_sendCtrlRequest('getBackupPlanDetails');
        $this->assertReplySuccessful($Reply);

        $plan = $Reply->getPayloadData();
        $this->assertNotEmpty($plan);
        $this->assertArrayHasKey('enabled', $plan, 'expected getBackupPlanDetails to have key "enabled"');
        $this->assertFalse($plan['enabled'], 'expected "enabled" to be false when plan is not assigned to the resource!');
    }

    /**
     * Test listCompatiblePlansCommitAction
     */
    public function testListCompatiblePlans()
    {
        echo __FUNCTION__;


        $Reply = $this->_sendCtrlRequest('listCompatiblePlans');
        $this->assertReplySuccessful($Reply);

        $plans = $Reply->getPayloadData();
        $this->assertNotEmpty($plans);

        foreach ($plans as $plan) {
            if ($plan['name'] === self::PLAN_NO_PRESCRIPT) {
                return $plan['id'];
            }
        }

        $this->fail('plan does not exist! Create in cloud and run tests again: ' . self::PLAN_NO_PRESCRIPT);
    }

    /**
     * test activatePlan with & without id specified.
     * Expects most recent plan to be activated if revoking
     * then activating without id specified
     *
     * @depends testListCompatiblePlans
     * @param string $expected_plan_id
     */
    public function testActivatePlanNoPrescript($expected_plan_id)
    {
        if (empty($expected_plan_id)) {
            $this->fail('FAIL');
        }
        echo __FUNCTION__;

        $Reply = $this->_sendCtrlRequest('activatePlan', array(
            Form_AbActivatePlan::INPUT_PLAN_ID => $expected_plan_id
        ));

        $this->assertReplySuccessful($Reply, "expected success activatePlanCommitAction!");

        $payloadData = $Reply->getPayloadData();
        $actual_id = $payloadData['id'];
        $actual_name = $payloadData['name'];
        $expected_name = self::PLAN_NO_PRESCRIPT;


        $this->assertTrue($payloadData['enabled'], 'expected plan to be enabled for resource!');
        $this->assertSame($expected_plan_id, $actual_id, "expected plan with id to be enabled: $expected_plan_id, instead found this id: $actual_id");
        $this->assertSame(self::PLAN_NO_PRESCRIPT, $actual_name);


        // make sure revoking / activating without id specified enabled the same plan
        $ReplyRevokePlan = $this->_sendCtrlRequest('revokePlan');
        $this->assertReplySuccessful($ReplyRevokePlan, "revokePlanCommitAction failed!");

        $ReplyActivateWithoutId = $this->_sendCtrlRequest('activatePlan');
        $this->assertReplySuccessful($ReplyActivateWithoutId, "Activate plan commit second time failed!!!");

        $activatedPlan = $this->_getActivatedPlan();

        $actual_id = trim($activatedPlan['id']);
        $actual_name = $activatedPlan['name'];

        $this->assertSame($expected_plan_id, $actual_id, "expected $expected_plan_id after second activation, instead this was activated: $actual_id");
        $this->assertSame($expected_name, $actual_name, "expected this name on activated plan after final activation: $expected_name, actual: $actual_name");
    }

    /**
     * Test activate default plan.
     */
    public function testActivateDefaultPlan()
    {
        echo __FUNCTION__;

        $Reply = $this->_sendCtrlRequest('activateInterWorxDefaultPlan');
        $this->assertReplySuccessful($Reply);

        $payloadData = $Reply->getPayloadData();
        $this->assertSame(Model_Backups_BackupPlan::DEFAULT_PLAN_NAME, $payloadData['name']);

        $activatedPlan = $this->_getActivatedPlan();
        $this->assertSame(Model_Backups_BackupPlan::DEFAULT_PLAN_NAME, $activatedPlan['name']);

        return $payloadData['id'];
    }


    /**
     * test getBackupPlanDetails.
     * Should get default plan after activating default plan.
     *
     * @depends testActivateDefaultPlan
     * @param string $expectedId
     */
    public function testGetBackupPlanDetails($expectedId)
    {
        echo __FUNCTION__;

        $Reply = $this->_sendCtrlRequest('getBackupPlanDetails');
        $this->assertReplySuccessful($Reply);

        $backupPlanDetails = $Reply->getPayloadData();
        $actualDays = $backupPlanDetails['days'];
        $actualName = $backupPlanDetails['name'];
        $actualId = $backupPlanDetails['id'];
        $actualEnabled = $backupPlanDetails['enabled'];

        $expectedName = Model_Backups_BackupPlan::DEFAULT_PLAN_NAME;
        $expectedCountDays = 7;

        $this->assertArrayHasKey('enabled', $backupPlanDetails);
        $this->assertArrayHasKey('id', $backupPlanDetails);
        $this->assertArrayHasKey('name', $backupPlanDetails);
        $this->assertArrayHasKey('time', $backupPlanDetails);
        $this->assertArrayHasKey('days', $backupPlanDetails);

        $this->assertTrue($actualEnabled, "expected plan to be enabled");
        $this->assertSame($expectedId, $actualId, "expected id: $expectedId");
        $this->assertSame($expectedName, $actualName, "expected name: $expectedName");
        $this->assertCount($expectedCountDays, $actualDays, "expected days count: $expectedCountDays");

        $time = $backupPlanDetails['time'];
        $actualHour = $time['hour'];
        $actualMinute = $time['minute'];

        $this->assertSame(2, $actualHour, "expected hour to be 2");
        $this->assertSame(0, $actualMinute, "expected minute to be 0");
    }

    /**
     * @return array
     */
    private function _getActivatedPlan()
    {
        $Reply = $this->_sendCtrlRequest('getBackupPlanDetails');
        return $Reply->getPayloadData();
    }
}
