<?php

/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2021 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @author     Malin Lorch <mlorch@liquidweb.com>
 * @date       Sep 10, 2021
 */
require_once(__DIR__ . '/../../Nodeworx.php');

/**
 * @package InterWorx
 * @subpackage
 */
class NwRecoverDatabasesTest extends Acronis_NodeworxTest
{
    /**
     * Disconnect db if connection created.
     */
    public function tearDown()
    {
        $dbConnection = $this->_getDbConnection();

        if (isset($dbConnection)) {
            $dbConnection->disconnect();
        }
    }

    /**
     * Test recoverDatabasesCommitAction for sw user.
     */
    public function testRecoverMappedDatabases()
    {
        echo __FUNCTION__;

        $this->_dropSwDbs();

        $Reply = $this->_recoverMappedDatabases(self::ACRONISTESTUSER);
        $this->assertReplySuccessful($Reply);

        $payloadData = $Reply->getPayloadData();
        $restoreId = $payloadData[AcronisReply::REPLY_RESTORE_ID];
        return $restoreId;
    }

    /**
     * Drop dbs
     */
    private function _dropSystemDb()
    {
        $Command = new Command('/home/interworx/bin/php /home/interworx/plugins/acronis/bin/acronisdbseeder.php --dropSystemDatabase');
        $Command->runAsRoot();
        $Command->exec();

        if (!$Command->wasSuccessful()) {
            // assume db doesn't exist?
            IW::getLog()->warning('Acronis Integration tests: Could not drop system db: ');
            IW::getLog()->warning($Command->getResultMessage());
        }
    }

    /**
     * @param string $unixuser
     * @return Reply
     */
    private function _recoverMappedDatabases($unixuser)
    {
        $Backup = $this->_getBackup();
        $dbs = $Backup->getUserDatabases($unixuser);

        $dbNames = array_map(function ($db) {
            return $db['name'];
        }, $dbs);

        $data = array(
            Form_AcronisBackups::INPUT_BACKUP_ID => $Backup->getBackupId(),
            Form_AcronisBackups::INPUT_DBS => $dbNames,
            Form_AcronisBackups::INPUT_USER => $unixuser

        );

        return $this->_sendCtrlRequest('restoreDatabases', $data);
    }

    /**
     * test recoverDatabases - should fail while other restore in progress
     */
    public function testRecoverDatabasesShouldFailWhileInProgress()
    {
        echo __FUNCTION__;
        $this->_skipIfNotInProgress();

        $Reply = $this->_recoverMappedDatabases(self::ACRONISTESTUSER);
        $this->assertReplyNotSuccessful($Reply, 'db recovery should fail when restore in progress');
    }

    /**
     * @param string $restoreId
     * @depends testRecoverMappedDatabases
     */
    public function testGetDatabaseRecoveryStateMappedDb($restoreId)
    {
        echo __FUNCTION__;
        $this->_testGetDbRecoveryState($restoreId);
    }

    /**
     * @param string $restoreId
     */
    private function _testGetDbRecoveryState($restoreId)
    {
        $data = array('id' => $restoreId);
        $Reply = $this->_sendCtrlRequest('getDatabaseRecoveryState', $data);
        $this->assertReplySuccessful($Reply);

        $payloadData = $Reply->getPayloadData();
        $this->assertArrayHasKey('id', $payloadData);
        $this->assertArrayHasKey('time', $payloadData);
        $this->assertArrayHasKey('databases', $payloadData);
        $this->assertArrayHasKey('unixuser', $payloadData);
        $this->assertArrayHasKey('state', $payloadData);
    }

    /**
     * @depends testRecoverMappedDatabases
     * @param string $restoreId
     */
    public function testListDatabaseRecoveries($restoreId)
    {
        echo __FUNCTION__;
        $this->_assertLatestRecoverySameId($restoreId);
    }

    /**
     * make sure that dbs exists after restoration.
     */
    public function testRecoverDatabasesRestoredMappedDbs()
    {
        echo __FUNCTION__;
        $this->_awaitDbRecoveries();
        $this->_assertSwDbsRestored();
    }

    /**
     * Test recoverDatabasesCommitAction for unmapped db.
     */
    public function testRecoverUnmappedDatabases()
    {
        echo __FUNCTION__;


        $this->_dropSystemDb();

        $Reply = $this->_recoverUnmappedDatabases();
        $this->assertReplySuccessful($Reply);

        $payloadData = $Reply->getPayloadData();
        $restoreId = $payloadData[AcronisReply::REPLY_RESTORE_ID];
        return $restoreId;
    }

    /**
     * @return Reply
     */
    private function _recoverUnmappedDatabases()
    {
        $Backup = $this->_getBackup();
        $dbNames = $this->_getUnmappedDbNames();

        $data = array(
            Form_AcronisBackups::INPUT_BACKUP_ID => $Backup->getBackupId(),
            Form_AcronisBackups::INPUT_DBS => $dbNames
        );

        return $this->_sendCtrlRequest('restoreDatabases', $data);
    }

    /**
     * test recoverDatabases - should fail while other restore in progress
     */
    public function testRecoverUnmappedDatabasesShouldFailWhileInProgress()
    {
        echo __FUNCTION__;
        $this->_skipIfNotInProgress();

        $Reply = $this->_recoverUnmappedDatabases();
        $this->assertReplyNotSuccessful($Reply, 'db recovery should fail when restore in progress');
    }

    /**
     * @param string $restoreId
     *
     * @depends testRecoverUnmappedDatabases
     */
    public function testGetDatabaseRecoveryStateUnmapped($restoreId)
    {
        echo __FUNCTION__;
        $this->_testGetDbRecoveryState($restoreId);
    }

    /**
     * @depends testRecoverUnmappedDatabases
     * @param string $restoreId
     */
    public function testListDatabaseRecoveriesShouldIncludeUnmapped($restoreId)
    {
        echo __FUNCTION__;
        $this->_assertLatestRecoverySameId($restoreId);
    }

    /**
     * @param string $restoreId
     */
    private function _assertLatestRecoverySameId($restoreId)
    {
        // last item is newest
        $recoveries = $this->_getDbRecoveries();
        $recovery = array_pop($recoveries);
        $this->assertSame($restoreId, $recovery['id']);
    }

    /**
     * make sure that dbs exists after restoration.
     */
    public function testRecoverDatabasesRestoredUnmappedDbs()
    {
        echo __FUNCTION__;
        $this->_awaitDbRecoveries();
        $this->_assertUnmappedDbsRestored();
    }

    /**
     * Make sure dbs exists & are populated after restore.
     */
    private function _assertUnmappedDbsRestored()
    {
        $unmappedDbs = $this->_getUnmappedDbNames();

        foreach ($unmappedDbs as $dbName) {
            $this->_assertUnmappedDbPopulated($dbName);
        }
    }


    /**
     * @param string $dbName
     */
    private function _assertUnmappedDbPopulated($dbName)
    {
        $this->_connectDb($dbName);

        $tableName = "systemdb";
        $this->_assertIdPopulatedInTable($tableName);
    }
}
